{% load sitetree %}
{% for item in sitetree_items %}
    {% if item.has_children %}
        <div class="ui dropdown {% if item.is_current or item.in_current_branch %}active{% endif %} item">{{ item.title_resolved }} <i class="dropdown icon"></i>
            {% sitetree_children of item for menu template "sitetree/menu_semantic_dropdown.html" %}
        </div>
    {% else %}
        <a href="{% sitetree_url for item %}" class="{% if item.is_current or item.in_current_branch %}active{% endif %} item">{{ item.title_resolved }}</a>
    {% endif %}
{% endfor %}