{% load sitetree %}
<div class="dropdown-menu" aria-labelledby="navitem-{{ item.id }}">
    {% for item in sitetree_items %}
        <a class="dropdown-item {% if item.is_current or item.in_current_branch %}active{% endif %}" href="{% sitetree_url for item %}" {% if item.hint %}title="{{ item.hint }}"{% endif %}>{{ item.title_resolved }}</a>
    {% endfor %}
</div>